#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

require 'yaml'
require 'fileutils'

class ChangeProxy
  
  CONFIG_PATH = '/vm.config'.freeze
  PROFILE_PATH = '/etc/profile.d/proxy.sh'.freeze

  private

  def check_nil(str)
    str ? str : ''
  end

  def show_proxy
    puts "Proxy Enable: #{@config['proxy_enable'].to_s}"
    if @config['proxy_enable'] then
      puts "Proxy Same HTTP/HTTPS: #{@config['proxy_same_http_https']}"
      fields = [ 'host', 'port', 'user', 'pass' ]
      fields.each do |field|
        value = @config["proxy_http_#{field}"]
        puts "Proxy HTTP #{field}: #{check_nil((field == 'pass' and value and value.size > 0) ? '***' : value )}"
      end
      if !@config['proxy_same_http_https'] then
        fields.each do |field|
          value = @config["proxy_https_#{field}"]
          puts "Proxy HTTPS #{field}: #{check_nil((field == 'pass' and value and value.size > 0) ? '***' : value )}"
        end
      end
    end
  end

  def save_config_file
    FileUtils.mv(CONFIG_PATH, CONFIG_PATH + '.old', :force => true)
    File.open(CONFIG_PATH, 'w') do |f|
      f.print YAML.dump(@config)
    end
    if @config['proxy_enable'] then
      File.open(PROFILE_PATH, 'w') do |f|
        if @config['proxy_same_http_https'] then
          host = @config['proxy_http_host']
          port = @config['proxy_http_port'].to_i
          user = @config['proxy_http_user']
          pass = @config['proxy_http_pass']
          user = nil if (user and user.size == 0)
          pass = nil if (pass and pass.size == 0)
          proxy_url = 'http://'
          if user and pass then
            proxy_url += "#{user}:#{pass}@"
          elsif user then
            proxy_url += "#{user}@"
          end
          proxy_url += "#{host}:#{port}/"
          f.puts "http_proxy=#{proxy_url}"
          f.puts "https_proxy=#{proxy_url}"
          f.puts "HTTP_PROXY=#{proxy_url}"
          f.puts "HTTPS_PROXY=#{proxy_url}"
          f.puts "export http_proxy https_proxy HTTP_PROXY HTTPS_PROXY"
        else
          http_host = @config['proxy_http_host']
          http_port = @config['proxy_http_port'].to_i
          http_user = @config['proxy_http_user']
          http_pass = @config['proxy_http_pass']
          http_user = nil if (http_user and http_user.size == 0)
          http_pass = nil if (http_pass and http_pass.size == 0)
          http_proxy_url = 'http://'
          if http_user and http_pass then
            http_proxy_url += "#{http_user}:#{http_pass}@"
          elsif http_user then
            http_proxy_url += "#{http_user}@"
          end
          http_proxy_url += "#{http_host}:#{http_port}/"
          if @config['proxy_https_host'] then
            https_host = @config['proxy_https_host']
            https_port = @config['proxy_https_port'].to_i
            https_user = @config['proxy_https_user']
            https_pass = @config['proxy_https_pass']
            https_user = nil if (https_user and https_user.size == 0)
            https_pass = nil if (https_pass and https_pass.size == 0)
            https_proxy_url = 'http://'
            if https_user and https_pass then
              https_proxy_url += "#{https_user}:#{https_pass}@"
            elsif https_user then
              https_proxy_url += "#{https_user}@"
            end
            https_proxy_url += "#{https_host}:#{https_port}/"
          else
            https_proxy_url = http_proxy_url
          end
          f.puts "http_proxy=#{http_proxy_url}"
          f.puts "https_proxy=#{https_proxy_url}"
          f.puts "HTTP_PROXY=#{http_proxy_url}"
          f.puts "HTTPS_PROXY=#{https_proxy_url}"
          f.puts "export http_proxy https_proxy HTTP_PROXY HTTPS_PROXY"
        end
      end
    else
      FileUtils.rm_f(PROFILE_PATH)
    end
  end

  def initialize(argv)
    @cmd = argv.shift
    @opts = argv

    begin
      @config = YAML.load(File.read(CONFIG_PATH))
    rescue Exception => e
      STDERR.puts "access failure: #{CONFIG_PATH}"
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end

    case @cmd
    when 'show_proxy' then
      show_proxy
    when 'proxy'
      flg = @opts ? @opts.first : nil
      if flg and /(enable|disable)/i =~ flg then
        flg = Regexp.last_match(1)
        if /enable/i =~ flg then
          @config['proxy_enable'] = true
        elsif /disable/i =~ flg then
          @config['proxy_enable'] = false
        end
      else
        STDERR.puts 'invalid argument'
        exit 1
      end
      save_config_file
      show_proxy
    when 'same_http_https'
      flg = @opts ? @opts.first : nil
      if flg and /(enable|disable)/i =~ flg then
        flg = Regexp.last_match(1)
        if /enable/i =~ flg then
          @config['proxy_same_http_https'] = true
        elsif /disable/i =~ flg then
          @config['proxy_same_http_https'] = false
        end
      else
        STDERR.puts 'invalid argument'
        exit 1
      end
      save_config_file
      show_proxy
    when 'set_http_proxy'
      if @opts.size < 2 or @opts.size > 4 then
        STDERR.puts 'invalid argument'
        exit 1
      end
      case @opts.size
      when 2 then
        @config['proxy_http_host'] = @opts[0]
        @config['proxy_http_port'] = @opts[1].to_i
      when 3 then
        @config['proxy_http_host'] = @opts[0]
        @config['proxy_http_port'] = @opts[1].to_i
        @config['proxy_http_user'] = @opts[2]
      when 4 then
        @config['proxy_http_host'] = @opts[0]
        @config['proxy_http_port'] = @opts[1].to_i
        @config['proxy_http_user'] = @opts[2]
        @config['proxy_http_pass'] = @opts[3]
      end
      save_config_file
      show_proxy
    when 'set_https_proxy'
      if @opts.size < 2 or @opts.size > 4 then
        STDERR.puts 'invalid argument'
        exit 1
      end
      case @opts.size
      when 2 then
        @config['proxy_https_host'] = @opts[0]
        @config['proxy_https_port'] = @opts[1].to_i
      when 3 then
        @config['proxy_https_host'] = @opts[0]
        @config['proxy_https_port'] = @opts[1].to_i
        @config['proxy_https_user'] = @opts[2]
      when 4 then
        @config['proxy_https_host'] = @opts[0]
        @config['proxy_https_port'] = @opts[1].to_i
        @config['proxy_https_user'] = @opts[2]
        @config['proxy_https_pass'] = @opts[3]
      end
      save_config_file
      show_proxy
    when 'unset_http_proxy'
      @config['proxy_http_host'] = nil
      @config['proxy_http_port'] = nil
      @config['proxy_http_user'] = nil
      @config['proxy_http_pass'] = nil
      save_config_file
      show_proxy
    when 'unset_https_proxy'
      @config['proxy_https_host'] = nil
      @config['proxy_https_port'] = nil
      @config['proxy_https_user'] = nil
      @config['proxy_https_pass'] = nil
      save_config_file
      show_proxy
    else
      STDERR.puts 'unknown command'
    end
  end

end

if $0 == __FILE__ then
  id = ChangeProxy.new(ARGV)
end
