#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

require 'yaml'

class RmDeletedHostsData

  BASE_PATH = '/var/lib/redmine'.freeze

  LOG_FILE = '/var/lib/redmine/log/rm_deleted_hosts_data.log'.freeze

  private

  def initialize
    run
  end

  def double_puts(*a)
    File.open(LOG_FILE, 'a') do |f|
      if a.respond_to?(:each) then
        a.each do |i|
          f.puts i
          STDERR.puts i
        end
      else
        f.puts a
        STDERR.puts a
      end
    end
  end

  def run
    begin

      double_puts "*** remove deleted hosts' data ***"

      double_puts "*** step1 (find deleted hosts) ***"
      deleted_host_ids = SidfmdHost.where(:flg_delete => true).collect {|x| x.id }.uniq
      double_puts " deleted_hosts=#{deleted_host_ids.inspect}"
      if deleted_host_ids.size == 0 then
        double_puts " not found deleted hosts"
        exit 1
      end

      double_puts "*** step2 (delete data) ***"
      deleted_host_ids.each do |deleted_host_id|
        SidfmdHost.transaction do
          host = SidfmdHost.where(:flg_delete => true).where(:id => deleted_host_id).first
          if host then
            hostid = host.id
            double_puts " #{deleted_host_id}:#{host.hostname} removing..."
            host.lock!
            unless host.flg_delete then
              double_puts " #{deleted_host_id}:#{host.hostname} is not deleted now. skipping..."
              next
            end
            host.last_checked_at = nil
            host.memo = nil
            host.is_agent = false
            host.save
            # カテゴリの削除
            SidfmdCategoriesHost.where(:sidfmd_host_id => hostid).delete_all
            # プロダクトの削除
            SidfmdHostApp.where(:sidfmd_host_id => hostid).delete_all
            # キューの削除
            sidfmd_queue_ids = SidfmdQueue.where(:sidfmd_host_id => hostid).collect {|x| x.id }.uniq
            sidfmd_queue_ids.each do |sidfmd_queue_id|
              SidfmdQueueDist.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
              SidfmdQueuePkg.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
              SidfmdQueueApp.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
              SidfmdQueueLapp.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
              SidfmdQueueWin.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
              sidfmd_queue_vul_ids = SidfmdQueueVul.where(:sidfmd_queue_id => sidfmd_queue_id).collect {|x| x.id }.uniq
              sidfmd_queue_vul_ids.each do |sidfmd_queue_vul_id|
                SidfmdQueueVulCve.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                SidfmdQueueVulSri.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                SidfmdQueueVulAffprod.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                SidfmdQueueVulAffver.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                SidfmdQueueVulAffpkg.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
              end
              SidfmdQueueVul.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
            end
            SidfmdQueue.where(:id => sidfmd_queue_ids).delete_all
            # チケット（脆弱性）の削除
            sidfmd_issue_ids = SidfmdIssue.where(:sidfmd_host_id => hostid).collect {|x| x.id }.uniq
            sidfmd_issue_ids.each do |sidfmd_issue_id|
              SidfmdIssueCve.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
              SidfmdIssueSri.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
              SidfmdIssueAffprod.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
              SidfmdIssueAffver.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
              SidfmdIssueAffpkg.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
            end
            SidfmdIssue.where(:id => sidfmd_issue_ids).delete_all
            double_puts " #{deleted_host_id}:#{host.hostname} removed."
          else
            double_puts " hostid #{deleted_host_id} is not found."
          end
          
          # for test: no-op
          #raise ActiveRecord::Rollback
        end
      end

      double_puts '*** FINISHED ***'
    rescue ActiveRecord::StatementInvalid => e
      if e.cause.is_a?(PG::UndefinedTable) then
        double_puts 'database not initialized'
        exit 1
      else
        double_puts 'Unknown Exception:'
        double_puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            double_puts i
          end
        end
      end
      exit 1
    rescue SystemExit => e
      double_puts "exit"
    rescue Exception => e
      double_puts 'Unknown Exception:'
      double_puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          double_puts i
        end
      end
      exit 1
    end
    exit 0
  end

end
