# -*- coding: utf-8 -*-

require 'sidfmdlm_api'
require 'sidfm_controller_support'

class SidfmInformationsController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  # 権限チェック
  def check_roles
    redirect_to home_path, :notice => '許可されていない操作です。' unless @sidfm_user
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? '/infos/admin/0' : "/infos/user/#{@user_category_ids.join(",")}"
  end

  public

  def index
    redirect_to :action => :vuln_list
  end

  def vuln_list
  end

  def vuln
    @id = params[:id].to_i
    if @id <= 0 then
      redirect_to :action => :vuln_list, :notice => 'idが不正です。' and return
    end
  end

  def product_index
  end

  def product
    @id = params[:id].to_i
    if @id <= 0 then
      redirect_to :action => :product_index, :notice => 'idが不正です。' and return
    end
  end

  def news
  end

  # プロダクトIDから登録済ホストを取得する
  # input: product_id (GET)
  # output: [ { :sidfmd_host_id => 123, :hostname => 'xxx' }, ... ]
  def api_prd2hosts
    product_id = params[:id].to_i
    if product_id <= 0 then
      render :layout => false, :json => [], :status => 400
      return
    end
    product = SidfmdProduct.find_by_pid(product_id)
    unless product then
      render :layout => false, :json => [], :status => 400
      return
    end
    versions = product.sidfmd_versions
    version_ids = versions.collect {|x| x.vid }
    user_host_ids_role = if @role_admin then
                           SidfmdHost
                             .joins(:sidfmd_categories_hosts)
                             .joins(:sidfmd_categories)
                             .where(:flg_delete => false)
                             .where(:sidfmd_categories => { :is_system => true })
                             .select(:id)
                             .distinct
                         else
                           @user_host_ids
                         end
    ret = []
    hostapps = SidfmdHostApp
      .joins(:sidfmd_host)
      .includes(:sidfmd_host)
      .where(:sidfmd_host_id => user_host_ids_role)
      .where("(sidfmd_host_apps.product_id = ? OR sidfmd_host_apps.version_id IN (?))", product_id, version_ids)
      .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
      .order('display_name')
      #.order("sidfmd_hosts.hostname")
    hostapps.each do |hostapp|
      temp_prod = SidfmdProduct.find_by_pid(hostapp.product_id)
      temp_ver = SidfmdVersion.find_by_vid(hostapp.version_id)
      if temp_ver and temp_ver.sidfmd_product then
        temp_ret = {
          :type => :version,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => temp_ver.sidfmd_product_id,
          :name => temp_ver.sidfmd_product.name,
          :version_id => hostapp.version_id,
          :version => temp_ver.version,
          :full_name => temp_ver.fullname,
          :is_support_end => temp_ver.is_support_end,
          :release_date => temp_ver.release_date,
          :support_date => temp_ver.support_date,
          :eol_date => temp_ver.eol_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      elsif temp_prod then
        temp_ret = {
          :type => :product,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => hostapp.product_id,
          :name => temp_prod.name,
          :is_support_end => temp_prod.is_support_end,
          :support_date => temp_prod.support_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      else
        # skip: invalid data
      end
    end
    # see vm_daemon
    queue_ids = SidfmdHost
      .where(:flg_delete => false)
      .where(:id => user_host_ids_role)
      .select("(SELECT id FROM sidfmd_queues WHERE sidfmd_hosts.id=sidfmd_queues.sidfmd_host_id ORDER BY created_at DESC LIMIT 1) AS lastest_queue_id")
    case product.name
    when 'Apache Struts' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Struts')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Tomcat' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Tomcat')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'PHP' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'PHP')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Ruby' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'ruby')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_versions.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Oracle Java SE SDK', 'Oracle Java SE JDK', 'Oracle Java SE JRE' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Oracle Java')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_version.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path, 
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    end
    ret.sort! {|x, y| x[:display_name] <=> y[:display_name] }
    render :layout => false, :json => ret
  end

  # 指定したhtml_idの配列から脆弱性詳細の有無を返す
  # input:  html_ids=JSON.stringify([ 1, 2, 3, ... ]) (POST)
  # output: { "1" => true, "2" => false, }
  # FIXME: under constructions
  def api_htmlids2issue
    htmlids = params[:html_ids]
    ret = {}
    unless htmlids then
      htmlids = []
    end
    unless htmlids.respond_to?(:collect) then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    begin
      htmlids = htmlids.collect {|x| x.to_i }.sort.uniq
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    if !htmlids.respond_to?(:each) or !htmlids.respond_to?(:'[]') then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    ret = SidfmdIssue
      .joins(:issue)
      .where(:sidfmd_host_id => if @role_admin then
                                  SidfmdHost
                                    .joins(:sidfmd_categories_hosts)
                                    .joins(:sidfmd_categories)
                                    .where(:flg_delete => false)
                                    .where(:sidfmd_categories => { :is_system => true })
                                    .select(:id)
                                    .distinct
                                else
                                  @user_host_ids
                                end)
      .where(:html_id => htmlids)
      .select(:html_id)
      .group(:html_id)
      .count
    render :layout => false, :json => ret.to_json
  end

end
