# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmUsersController < ApplicationController

  unloadable

  layout 'sidfm'

  self.main_menu = false

  before_action :check_roles

  protect_from_forgery with: :exception
  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :create_common_data
  before_action :get_sidfm_roles
  before_action :prevent_access
  before_action :check_roles, except: [:index, :modify, :test_email] 
  before_action :get_admin_ids

  # 権限チェック
  def prevent_access
    get_sidfm_roles
    unless @sidfm_user 
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end 
  end

  def check_roles
    if !@sidfm_roles.include?(:admin) and !@sidfm_roles.include?(:use) then
      redirect_to :action => :index, :notice => '許可されていない操作です。'
    end
  end

  def get_admin_ids
    @sidfm_admin = Role.find_by(name: 'SIDfmVM管理者').id
    @admin = Role.find_by(name: '管理者').id
  end

  #ソート処理
  def sort_for_users
    order_param = @users_paginate[:order].split('_')
    col = order_param[0]
    order = order_param[1]
    case col
    when 'loginname' then
      col = 'login'
    when 'lastname' then
      col = 'lastname'
    when 'authway' then
      col = 'auth_source_id'
    when 'mail-notification' then
      col = 'mail_notification'
    end
    case order
    when 'asc'
      @users_paginate[:content] = @users_paginate[:content].order(col => :asc).order(:login => :asc)
    when 'desc'
      @users_paginate[:content] = @users_paginate[:content].order(col => :desc).order(:login => :desc)
    end
    #全ユーザのidをoffset前に取得
    @all_filtered_users = @users_paginate[:content].where(:admin => false)
    @users_paginate[:content] = @users_paginate[:content].offset(@users_paginate[:offset]).limit(@users_paginate[:per_page])
  end
  
  #検索処理
  def search_items
    logins = @users_paginate[:filters][:search_login]
    names = @users_paginate[:filters][:search_name]
    auths = @users_paginate[:filters][:search_auth]
    users_auths = @users_paginate[:filters][:search_users_auth] 
    category = @users_paginate[:filters][:search_category]
    self_user = @users_paginate[:filters][:search_self_user]
    notification = @users_paginate[:filters][:search_mail_notification]

    if !logins.nil? && logins.length > 0 then
      if logins != User.current.login then
        @users_paginate[:content] = @users_paginate[:content].where('login LIKE ?', "%#{logins}%")
      else
        @users_paginate[:content] = @users_paginate[:content].where(:login => logins)
      end
    end

    if !names.nil? && names.length > 0 then
      @users_paginate[:content] = @users_paginate[:content].where('firstname LIKE ? OR lastname LIKE ?', "%#{names}%", "%#{names}%")
    end
    
    if !auths.nil? && auths.length > 0 then
      auth = auths.to_i
      if auth == -1 then
        @users_paginate[:content] = @users_paginate[:content].where(auth_source_id: nil)
      else
        @users_paginate[:content] = @users_paginate[:content].joins("LEFT OUTER JOIN auth_sources ON users.auth_source_id = auth_sources.id").where("auth_sources.id = ?", auth)
      end
    end

    if !users_auths.nil? && users_auths.length > 0 then
      auth = users_auths.to_i
      if auth == -1 then
        @users_paginate[:content] =  @users_paginate[:content].where.not(:admin => true).where.not(id: Member.all.map{|x| x.user_id})
      else
        @users_paginate[:content] =  @users_paginate[:content].joins("LEFT OUTER JOIN members ON project_id = #{@sidfm_project.id} AND members.user_id=users.id").joins("LEFT OUTER JOIN member_roles ON members.id = member_roles.member_id").joins("LEFT OUTER JOIN roles ON member_roles.role_id = roles.id").where("roles.id = ?", auth)
      end
    end

    if !category.nil? && category.length > 0 then
      category = category.to_i
      admin_userids = User.joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id").joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id").joins("LEFT OUTER JOIN roles ON roles.id = member_roles.role_id").where("roles.id = ?", @sidfm_admin).map(&:id)
      @users_paginate[:content] = @users_paginate[:content].joins("LEFT OUTER JOIN sidfmd_categories_users ON sidfmd_categories_users.user_id = users.id").joins("LEFT OUTER JOIN sidfmd_categories ON sidfmd_categories_users.sidfmd_category_id = sidfmd_categories.id").where("sidfmd_categories.id = ?", category).where.not(id: admin_userids)
    end

    if !notification.nil? && notification.length > 0 then
      if notification == "true" then
        @users_paginate[:content] = @users_paginate[:content].where(:mail_notification => "only_my_events")
      elsif notification == "false" then
        @users_paginate[:content] = @users_paginate[:content].where(:mail_notification => "none")
      end
    end
  end
     
  public
 
  def index
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked) 
    @users_paginate = {}
    @users_paginate[:per_page] = (filtered_params[:per_page] || 10).to_i 
    @users_paginate[:offset] = (filtered_params[:offset] || 0).to_i
    @users_paginate[:filters] = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    #ソート・検索機能
    @users_paginate[:search_login] = @users_paginate[:filters][:search_login] || ''
    @users_paginate[:search_name] = @users_paginate[:filters][:search_name] || ''
    @users_paginate[:search_auth] = @users_paginate[:filters][:search_auth] || ''
    @users_paginate[:search_users_auth] = @users_paginate[:filters][:search_users_auth] || ''
    @users_paginate[:search_category] = @users_paginate[:filters][:search_category] || ''
    @users_paginate[:search_mail_notification] = @users_paginate[:filters][:search_mail_notification] || ''
    #チェックボックス
    @users_paginate[:checked] = JSON.parse(filtered_params[:checked] || '[]')
    @users_paginate[:all_checked] = ((filtered_params[:all_checked] || '') == 'true')
   
    @users_paginate[:content] = User.where.not(:type => 'AnonymousUser')
    
    #管理者以外はカテゴリは自分のカテゴリに関係するもののみ表示
    if @sidfm_roles.include?(:admin) then
      users_categories = SidfmdCategory.where(:is_system => false).sort_by{|c| c.name}
      ccids = users_categories.map{|c| c.id}
      @categories = Hash[users_categories.map{ |cat| [cat.id, cat.name]}]
    elsif !@sidfm_roles.include?(:admin) && User.current.sidfmd_categories.size > 0 then 
      users_categories = User.current.sidfmd_categories.where(:is_system => false).sort_by{|c| c.name}
      @categories = Hash[users_categories.map{ |cat| [cat.id, cat.name]}]
      ccids = users_categories.collect {|x| x.id }
      userids = SidfmdCategoriesUser.where(:sidfmd_category_id => ccids).collect {|x| x.user_id }
      admin_userids = @users_paginate[:content].joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id").joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id").joins("LEFT OUTER JOIN roles ON roles.id = member_roles.role_id").where("roles.id = ?", @sidfm_admin).map(&:id)
      userids_except_admin = userids - admin_userids
      @users_paginate[:content] = @users_paginate[:content].where(:id => userids_except_admin).where(:admin => false)
    #elsif !@sidfm_roles.include?(:admin) && User.current.sidfmd_categories.size == 0 then
    else
      ccids = []
      @categories = {}
      @users_paginate[:content] = @users_paginate[:content].where(:id => User.current) 
    end
    #ユーザの権限。カテゴリ一括更新の時、管理者権限のユーザを対象から除外するために使用する
    roles_and_users = Role.joins(:member_roles).joins(:members).joins("LEFT OUTER JOIN users ON users.id = members.user_id")
    @users_role_names = {}
    @users_paginate[:content].each do |user|
      user_role = roles_and_users.where("users.id = ?", user).first  
      @users_role_names[user.id] = user_role.name if user_role
    end

    @all_users = @users_paginate[:content].where(:admin => false)
    search_items
    @users_paginate[:order] = (filtered_params[:order] || 'loginname_asc')
    sort_for_users
    @users_paginate[:total_size] = @users_paginate[:content].except(:order, :offset, :limit).count
    #ユーザ毎の権限（:admin, :use, :view)
    @users_perms = {}
    #表示されるユーザの取得
    @users = @users_paginate[:content]

    #カテゴリ
    @current_user_category = ccids
    @user_category_ids = {}
    @users_categories ={}
    #権限とプロジェクトの取得
    @auths = AuthSource.all
    @users_auths = {}
    @users_auths_id ={}
    @roles_for_users = []
    Role.where(:name => ['SIDfmVM管理者','SIDfmVM報告者', 'SIDfmVM閲覧者']).each do |role|
      if role.name == 'SIDfmVM管理者' then
        @roles_for_users[0] = role
      elsif role.name == 'SIDfmVM報告者' then
        @roles_for_users[1] = role
      elsif role.name == 'SIDfmVM閲覧者' then
        @roles_for_users[2] = role    
      end
    end                  
    @role_names = {}
    @role_id = {}
    @current_user_roleid = roles_and_users.where("users.id = ?", User.current).first.id
    @sidfm_view_id = Role.find_by(name: 'SIDfmVM閲覧者').id
    #メール通知設定
    @mail_notification = Hash[User.all.map(&:mail_notification).uniq.map{|v| v == "none" ? ["無効", "false"] : ["有効", "true"]}]
    #ユーザ毎に情報を取得・設定
    @users.each do |user|
      #ユーザのカテゴリを取得
      users_cats = user.sidfmd_categories.where(:id => ccids)
      @user_category_ids[user] = users_cats.map{|c| c.id}
      @users_categories[user] = Hash[users_cats.map{|c| [c.id, c.name]}]
      #ユーザの認証方式の取得
      if user.auth_source_id.nil? then
        @users_auths_id[user] = -1
      else
        @users_auths_id[user] = user.auth_source.id
        @users_auths[user] = user.auth_source.name
      end
        
      #ユーザの権限(:admin, :user, :view)取得       
      role_for_user = get_role(user)

      #ユーザのプロジェクト権限取得
      user_role = roles_and_users.where("users.id = ?", user).first
      if user_role.nil? then
        @role_id[user] = -1
      else
        @role_id[user] = user_role.id
        @role_names[user] = user_role.name
      end
    end
    #ブラウザバックかどうか判断するための処理
    if !session['chkbok_ids'] then
      session['chkbok_ids'] = {}
    end
    if !session['chkbok_ids']['user'] then
      session['chkbok_ids']['user'] = {}
    end
    session['chkbok_ids']['user']['is_browser_back'] = false
  end

  def create
    filtered_params = params.fetch(:form, {}).permit(:userid, :login, :pw, :pwconf, :lastname, :firstname, :email, :auth, :pj_auth, :mail_notification)
    begin
      User.transaction do
        p_auth = params[:form][:auth].to_i
        p_pj_auth = params[:form][:pj_auth].to_i
        p_mail_notification = if params[:form][:mail_notification].nil? then
                                "none"
                              elsif params[:form][:mail_notification] == "true" then
                                "only_my_events"
                              else 
                                redirect_to :action => :index, :notice => '不正な操作です' and return
                              end
        checked = validate_fetch_params('cre')
        if !checked[:success] then
          redirect_to :action => :index, :notice => checked[:notice] and return
        end
        user_admin = User.new
        user_admin.login = checked[:p_name]
        user_admin.password = checked[:p_pw]
        user_admin.password_confirmation = checked[:p_pwconf]
        user_admin.lastname = checked[:p_lastname]
        user_admin.firstname = checked[:p_firstname]
        user_admin.mail = checked[:p_email]
        user_admin.mail_notification = p_mail_notification
        user_admin.sidfmd_categories = User.current.sidfmd_categories unless @sidfm_roles.include?(:admin)
        unless p_auth == -1 then
          user_admin.auth_source_id = p_auth
        end
          
        if user_admin.save then
          #ユーザ権限の登録
          if p_pj_auth == @admin then
            redirect_to :action => :index, :notice => '管理者権限は付与できません。' and return
          elsif p_pj_auth != -1 then
            newmem = Member.create_principal_memberships(user_admin, :project_ids => [ @sidfm_project.id ], :role_ids => p_pj_auth)
            if newmem then
              redirect_to :action => :index and return
            else
              redirect_to :action => :index, :notice => 'ユーザ権限の登録に失敗しました。' and return
            end
          else
            redirect_to :action => :index, :notice => 'ユーザ権限の登録が必要です。' and return
          end
        else
          redirect_to :action => :index, :notice => user_admin.errors.full_messages.join('。') + "。" and return              
        end
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'ユーザの登録に失敗しました。' and return
    end
    redirect_to :action => :index and return
  end

  def modify
    filtered_params = params.fetch(:form, {}).permit(:userid, :login, :pw, :pwconf, :lastname, :firstname, :email, :auth, :pj_auth, :per_page, :filters, :offset, :order, :mail_notification) 
    begin
      User.transaction do
        user = User.lock.find(filtered_params['userid'].to_i)
        if user then
          p_auth = params[:form][:auth].to_i
          p_pj_auth = params[:form][:pj_auth].to_i
          p_mail_notification = if params[:form][:mail_notification].nil? then
                                  "none"
                                elsif params[:form][:mail_notification] == "true" then
                                  "only_my_events"
                                else
                                  redirect_to :action => :index, :notice => '不正な操作です' and return
                                end
          unless @sidfm_roles.include?(:admin) || @sidfm_roles.include?(:use) then
            sidfm_view_id = Role.find_by(name: 'SIDfmVM閲覧者').id
            user_authsource_id = User.current.auth_source_id 
            user_authsource_id = -1 if user_authsource_id.nil?
            unless user == User.current && p_auth == user_authsource_id && p_pj_auth == sidfm_view_id then
              redirect_to :action => :index, :notice => 'ユーザの更新は許可されていません。' and return
            end
          end       
          checked = validate_fetch_params('mod')
          if !checked[:success] then
            redirect_to :action => :index, :notice => checked[:notice] and return
          end
          if user.login == 'sidfmadm' && checked[:p_name] != user.login then
            redirect_to :action => :index, :notice => 'ユーザの更新は許可されていません。' and return
          end
          user.login = checked[:p_name]
          user.lastname = checked[:p_lastname]
          user.firstname = checked[:p_firstname]
          user.mail = checked[:p_email]
          user.mail_notification = p_mail_notification
          unless checked[:p_pw] == '' then
            user.password = checked[:p_pw]
            user.password_confirmation = checked[:p_pwconf]
          end
          if p_auth <= 0 then
            user.auth_source_id = nil
          else
            user.auth_source_id = p_auth
          end
          opts ={}
          opts[:action] = :index
          if @sidfm_roles.include?(:admin) then
            opts[:'pages[offset]'] = filtered_params[:offset].to_i
            opts[:'pages[per_page]'] = filtered_params[:per_page]
            opts[:'pages[filters]'] = filtered_params[:filters] || '{}'
            opts[:'pages[order]'] = filtered_params[:order]
          end
          admin_users = User.joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id").joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id").where("member_roles.role_id = ?", @sidfm_admin)
          if (p_pj_auth != @sidfm_admin && admin_users.include?(user) && admin_users.size == 1) || (p_pj_auth == -1 && checked[:p_id] == User.current.id) || (user.login == 'sidfmadm' && p_pj_auth != @sidfm_admin) then
            redirect_to :action => :index, :notice => 'このユーザの権限は変更できません。' and return
          end
          role_checked = 0
          if p_pj_auth == @admin && user.admin == false then
            redirect_to :action => :index, :notice => '管理者権限は付与できません。' and return
          elsif p_pj_auth != -1 then
            role_checked = role_check(checked[:p_id])
            if role_checked != -2 then
              unless MemberRole.find(role_checked).update_attribute :role_id, p_pj_auth
                redirect_to :action => :index, :notice => '権限の登録に失敗しました。' and return
              end
            end
          elsif p_pj_auth == -1 then
            user.sidfmd_categories = []
          end

          if user.save then
            if p_pj_auth == -1 then
              if member_del = Member.where(:project_id => @sidfm_project.id).joins(:user).where("users.id = ?", checked[:p_id]).first
                unless  member_del.destroy then
                  redirect_to :action => :index, :notice => '権限の削除に失敗しました。' and return
                end
              end 
            elsif role_checked == -2 then 
              newmem = Member.create_principal_memberships(user, :project_ids => [ @sidfm_project.id ], :role_ids => p_pj_auth) 
              unless newmem then
                redirect_to :action => :index, :notice => '権限の登録に失敗しました。' and return
              end
            end 
            if user.id == User.current.id
              # The session token was destroyed by the password change, generate a new one
              session[:tk] = user.generate_session_token
            end
            redirect_to opts and return
          else 
            redirect_to :action => :index, :notice => user.errors.full_messages.join('。') + "。" and return
          end
        else         
          redirect_to :action => :index, :notice => '対象のユーザが見つかりません。' and return
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'ユーザ情報の更新に失敗しました。' and return
    end
    redirect_to :action => :index, :notice => 'ユーザ情報の更新に失敗しました。' and return
  end

  def modify_category 
    filtered_params = params.fetch(:categories, {}).permit(:userid, :selected_category_ids, :offset, :per_page, :filters, :order, :rule_info)
    begin
      rule_info = JSON.parse(filtered_params[:rule_info] || '{}').symbolize_keys
    rescue JSON::ParserError => e
      log_exception(e)
      redirect_to :action => :index, :notice => '不正な入力です。' and return
    end
    begin
      p_userid = filtered_params['userid']
      p_categories = JSON.parse(filtered_params['selected_category_ids']).map(&:to_i)
      mod_categories = p_categories.map{|x| SidfmdCategory.find(x)}
      mod_categories.compact!
      @users_perms = {}
      User.transaction do
        #関連するルールの削除・変更
        delete_rule_ids = rule_info[:delete_rule_ids]
        delete_action_ids = rule_info[:delete_action_ids]
        if delete_rule_ids && delete_rule_ids.size > 0 then
          SidfmdRule.where(:id => delete_rule_ids).destroy_all
        end
        if delete_action_ids && delete_action_ids.size > 0 then
          SidfmdRuleAction.where(:id => delete_action_ids).destroy_all
        end

        user = User.find(p_userid)
        get_role(user)
        unless @users_perms[user].include?(:admin) || user.members.where('project_id = ?', @sidfm_project.id).size == 0 || (!@sidfm_roles.include?(:admin) && user == User.current) then
          if @sidfm_roles.include?(:admin) then
            user.sidfmd_categories = mod_categories
          else
            unchanged_categories = user.sidfmd_categories - User.current.sidfmd_categories
            user_categories =  mod_categories + unchanged_categories
            user_categories.uniq!
            user.sidfmd_categories = user_categories.compact
          end
        else
          redirect_to :action => :index, :notice => 'このユーザのカテゴリは変更できません。' and return
        end
        user.save
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'カテゴリー情報の更新に失敗しました。' and return  
    end
    opts = {}
    opts[:action] = :index
    if @sidfm_roles.include?(:admin) then
      opts[:'pages[offset]'] = filtered_params[:offset].to_i
      opts[:'pages[per_page]'] = filtered_params[:per_page]
      opts[:'pages[filters]'] = filtered_params[:filters] || '{}'
      opts[:'pages[order]'] = filtered_params[:order]
    end
    redirect_to opts and return
  end

  def update_authenticate_bulk
    filtered_params = params.fetch(:authenticate, {}).permit(:userids, :auth, :dump, :offset, :per_page, :filters, :order)
    userids = JSON.parse(filtered_params[:userids] || '[]')
    auth = filtered_params[:auth]
    dump = filtered_params[:dump]
    offset = filtered_params[:offset]
    per_page = filtered_params[:per_page]
    filters = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    order = filtered_params[:order]
    update_users(userids, auth, dump, "authentication", offset, per_page, filters, order)
  end

  def update_authority_bulk
    filtered_params = params.fetch(:authority, {}).permit(:userids, :auth, :dump, :offset, :per_page, :filters, :order)
    userids = JSON.parse(filtered_params[:userids] || '[]')
    auth = filtered_params[:auth]
    if (!@sidfm_roles.include?(:admin) && auth.to_i == @sidfm_admin) || auth.to_i == @admin then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    admin_users = User.joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id").joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id").where("member_roles.role_id = ?", @sidfm_admin)
    admin_users_size = admin_users.size
    bulk_admin_users_size = admin_users.where(id: userids).size
    if auth.to_i != @sidfm_admin && admin_users_size == bulk_admin_users_size then 
      redirect_to :action => :index, :notice => '管理者を0にはできません。' and return
    end
    dump = filtered_params[:dump]
    offset = filtered_params[:offset]
    per_page = filtered_params[:per_page]
    filters = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    order = filtered_params[:order]
    update_users(userids, auth, dump, "authority", offset, per_page, filters, order)
   
  end
    
  def update_categories_bulk
    filtered_params = params.fetch(:categories_bulk, {}).permit(:userids, :selected_category_ids, :dump, :offset, :per_page, :filters, :order, :rule_info)
    dump = {}
    begin
      dump = JSON.parse(filtered_params[:dump])
      userids = JSON.parse(filtered_params[:userids])
      selected_category_ids = JSON.parse(filtered_params[:selected_category_ids])
      rule_info = JSON.parse(filtered_params[:rule_info] || '{}').symbolize_keys
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:action] = :index
      dump[:notice] = '入力が不正です。'
      redirect_to dump and return
    end
    dump[:action] = :index
    unless userids or selected_category_ids then
      dump[:notice] = '入力が不正です。'
    end
    @users_perms = {}    
    begin
      User.transaction do
        #関連するルールの削除・変更
        delete_rule_ids = rule_info[:delete_rule_ids]
        delete_action_ids = rule_info[:delete_action_ids]
        if delete_rule_ids && delete_rule_ids.size > 0 then
          SidfmdRule.where(:id => delete_rule_ids).destroy_all
        end
        if delete_action_ids && delete_action_ids.size > 0 then
          SidfmdRuleAction.where(:id => delete_action_ids).destroy_all
        end

        User.where(id: userids.map(&:to_i)).each{|user|
          get_role(user)
          unless @users_perms[user].include?(:admin) || @users_perms[user].size == 0 ||(!@sidfm_roles.include?(:admin) && user == User.current) then
            update_categories = SidfmdCategory.where(id: selected_category_ids.map(&:to_i))
            if @sidfm_roles.include?(:admin) then 
              user.sidfmd_categories = update_categories
            else
              unchanged_categories = user.sidfmd_categories - User.current.sidfmd_categories
              update_categories = update_categories + unchanged_categories
              update_categories.uniq!
              user.sidfmd_categories = update_categories.compact
            end
          else 
            redirect_to :action => :index, :notice => 'カテゴリを変更できないユーザが含まれています。' and return
          end
          unless user.save then
            dump[:notice] = 'カテゴリの更新に失敗しました。'
            redirect_to dump
          end
        }
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => e and return
    end
    if @sidfm_roles.include?(:admin) then
      dump[:'pages[offset]'] = filtered_params[:offset].to_i
      dump[:'pages[per_page]'] = filtered_params[:per_page]
      dump[:'pages[filters]'] = filtered_params[:filters] || '{}'
      dump[:'pages[order]'] = filtered_params[:order]
    end
    redirect_to dump and return
  end
  
  def update_mail_notification_bulk
    filtered_params = params.fetch(:mail_notification_bulk, {}).permit(:userids, :notification, :dump, :offset, :per_page, :filters, :order)
    dump = {}
    begin
      dump = JSON.parse(filtered_params[:dump])
      userids = JSON.parse(filtered_params[:userids])
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:action] = :index
      dump[:notice] = '入力が不正です。'
      redirect_to dump and return
    end
    dump[:action] = :index
    unless userids then
      dump[:notice] = '入力が不正です。'
    end
    p_mail_notification = if filtered_params[:notification] == "false" then
                            "none"
                          elsif filtered_params[:notification] == "true" then
                            "only_my_events"
                          else
                            redirect_to :action => :index, :notice => '不正な操作です' and return
                          end
    @users_perms = {}
    begin
      User.transaction do
        users = User.where(id: userids.map(&:to_i))
        users.each do |user|
          get_role(user)
          if !@sidfm_roles.include?(:admin) && @users_perms[user].include?(:admin) then
            redirect_to :action => :index, :notice => '不正な操作です' and return
          end
        end
        users.update_all(mail_notification: p_mail_notification)
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => e and return
    end
    if @sidfm_roles.include?(:admin) then
      dump[:'pages[offset]'] = filtered_params[:offset].to_i
      dump[:'pages[per_page]'] = filtered_params[:per_page]
      dump[:'pages[filters]'] = filtered_params[:filters] || '{}'
      dump[:'pages[order]'] = filtered_params[:order]
    end
    redirect_to dump and return
  end

  def delete
    filtered_params = params.fetch(:formdel, {}).permit(:id, :rule_info)
    userid = filtered_params['id'].to_i
    user_del = User.find_by(id: userid)
    roles_and_users = Role.joins(:member_roles).joins(:members).joins("LEFT OUTER JOIN users ON users.id = members.user_id")
    unless roles_and_users != nil then
      roleid = roles_and_users.where("users.id = ?", user_del).first.id
    end
    if user_del.admin == true || (roleid == @sidfm_admin && roles_and_users.where("roles.id = ?", @sidfm_admin).size == 1) || user_del == User.current then
      redirect_to :action => :index, :notice => 'このユーザは削除できません。' and return
    end
    begin
      rule_info = JSON.parse(filtered_params[:rule_info] || '{}').symbolize_keys
    rescue JSON::ParserError => e
      log_exception(e)
      redirect_to :action => :index, :notice => '不正な入力です。' and return
    end
    begin 
      User.transaction do
        #関連するルール・メール送信の操作・メール送信の送信先ユーザの削除
        delete_rule_ids = rule_info[:delete_rule_ids]
        delete_action_ids = rule_info[:delete_action_ids]
        if delete_rule_ids && delete_rule_ids.size > 0 then
          SidfmdRule.where(:id => delete_rule_ids).destroy_all
        end
        if delete_action_ids && delete_action_ids.size > 0 then
          SidfmdRuleAction.where(:id => delete_action_ids).destroy_all
        end
        if user_del.destroy then
          Rails.cache.clear
          redirect_to :action => :index
          return
        end
      end
    rescue Exception => e 
      log_exception(e)
      redirect_to :action => :index, :notice => 'ユーザの削除に失敗しました。' and return  
    end
  end
  
  def test_email
    filtered_params = params.fetch(:form, {}).permit(:email_address, :subject, :message)
  begin 
    mail = {}
    #送信者はsidfmdadm
    mail_sender_address = User.find_by(:login => 'sidfmadm').mail
    mail[:from] = mail_sender_address
    mail[:to] = filtered_params[:email_address]
    mail[:subject] = filtered_params[:subject]
    mail[:message] = filtered_params[:message]
    SidfmdMailer.send_mail(mail).deliver_now 
  rescue Exception => e
    log_exception(e)
    redirect_to :action => :index, :notice => e and return
  end
    redirect_to :action => :index and return
  end

  private

  def get_role(user)
    roles = user.roles_for_project(get_sidfm_project)
    temp = []
    roles.each do |role|
      perms = role.permissions
      if perms then
        temp << :admin if perms.include?(:admin_sidfm)
        temp << :use if perms.include?(:use_sidfm)
        temp << :view if perms.include?(:view_sidfm)
      end
    end
    temp.uniq!
    @users_perms[user] = temp
  end


  def update_users(userids, auth, u_dump, method, offset, per_page, filters, order)
   dump ={}
   begin
     dump = JSON.parse(u_dump)
   rescue JSON::ParserError => e
     log_exception(e)
     dump[:action] = :index
     dump[:notice] = '入力が不正です。'
     redirect_to dump and return
   end
   dump[:action] = :index
   auth = auth.to_i
   unless userids or auth then
     dump[:notice] = '入力が不正です。'
   end
   if auth.nil? || auth == 0 then
     dump[:notice] = '入力が不正です。'
   end
   if auth < 0 then
     auth = nil
   end
   begin
     User.transaction do
       userids.each do |i|
         id = i.to_i
         user = User.find(id)
         if user then 
           if method == "authentication" then
             user.update_attribute :auth_source_id, auth
           elsif method == "authority" then 
             if user.login == 'sidfmadm' then
               redirect_to :action => :index, :notice => 'このユーザの権限は変更できません。' and return
             end
             roleid =  role_check(id)
             if roleid then
               if roleid == -2 then
                 newmem = Member.create_principal_memberships(user, :project_ids => [ @sidfm_project.id ], :role_ids => auth)
               else
                 memrole = MemberRole.find(roleid)
                 memrole.update_attribute :role_id, auth
               end        
             else
               redirect_to :action => :index, :notice => '権限の更新に失敗しました。' and return
             end
           end
         end
       end
     end
   rescue Exception => e
     log_exception(e)
     redirect_to :action => :index, :notice => '権限の更新に失敗しました。' and return
   end
   if @sidfm_roles.include?(:admin) then
     dump[:'pages[offset]'] = offset.to_i
     dump[:'pages[per_page]'] = per_page
     dump[:'pages[filters]'] = JSON.generate(filters)
     dump[:'pages[order]'] = order
   end
   redirect_to dump
  end

  def validate_fetch_params(method)
    unless params[:form].is_a?(Hash)
      return nil
    end
    p_id = params[:form][:userid].to_i
    p_name = params[:form][:login]
    p_pw = params[:form][:pw]
    p_pwconf = params[:form][:pwconf]
    p_lastname = params[:form][:lastname]
    p_firstname = params[:form][:firstname]
    p_email = params[:form][:email]
    func = method
    if p_name.delete!('\\|/[]^~*:#!"+,\'<>') then
      return { :sccess => false, :notice => 'ログイン名に使用できない文字が含まれています。'}
    end
    if func == 'cre' && (p_pw.nil? or p_pw.size == 0) then
      return { :success => false, :notice => 'パスワードが入力されていません。' }
    end
    if func == 'cre' && (p_pwconf.nil? or p_pwconf.size == 0) then
      return { :success => false, :notice => 'パスワード確認を入力して下さい。'}
    elsif p_pwconf != p_pw then
      return { :success => false, :notice => 'パスワードが一致しません。'}
    end
    if p_lastname.nil? or p_name.size == 0 then
      return { :success => false,  :notice => '名字が入力されていません。'}
    end
    if p_firstname.nil? or p_firstname.size == 0 then
      return { :success => false, :notice => '名前が入力されていません。'}
    end

    if p_name.size > 60 then
      return { :success => false, :notice => 'ログイン名は60文字以内で入力してください。'}
    end
 
    if !p_email.match(/\A[a-zA-Z0-9_\#!$%&`'*+\-{|}~^\/=?\.]+@[a-zA-Z0-9][a-zA-Z0-9\.-]+\z/) then
      return { :success => false, :notice => 'メールアドレスに誤りがあります。'}
    end

    unless p_pw == '' && func == 'mod' then
      if p_pw.size < Setting.password_min_length.to_i then
        return { :success => false,  :notice => 'パスワードは8文字以上で入力してください。'}
      end
    end

    if p_lastname.size > 30 then
      return { :success => false, :notice => '姓は30文字以内で入力してください。'}
    end
    if p_firstname.size > 30 then
      return { :success => false, :notice => '名は30文字以内で入力してください。'}
    end
    if func == 'mod' then
      same_login_user = User.find_by(login: p_name)
      if same_login_user then
        unless same_login_user.id == p_id then
          return { :success => false, :notice => 'ログイン名は既に使用されています。'}
        end
      end
    else
      if User.find_by(login: p_name) then
        return { :success => false, :notice => 'ログイン名は既に使用されています。'}
      end
    end

    { :success => true, :p_id => p_id, :p_name => p_name, :p_pw => p_pw, :p_pwconf => p_pwconf, :p_lastname => p_lastname, :p_firstname => p_firstname, :p_email => p_email}
  end

  def role_check(p_id)
    memofuser = Member.where(:project_id => @sidfm_project.id).find_by(user_id: p_id)
    unless memofuser then
      return -2
    end
    member_role_id = MemberRole.joins(:member).where("member_id = ?", memofuser).first.id
  end
end
