#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require 'time'

require 'softek_utils'
require 'vm_config'
require 'sidapi'

class Sidfm < ActiveRecord::Base
  unloadable

  def self.get_value(key)
    r = where(:key => key)
    r ? (r.first ? r.first.value : nil) : nil
  end

  def self.set_value(key, value)
    ret = nil
    r = where(:key => key)
    if r and r.first then
      ret = r.first.update(:value => value)
    else
      ret = self.create(:key => key, :value => value)
    end
    ret
  end

  # FIXME: zabbix_*の整理(zabbix_senderが使える最小限に)
  # FIXME: sidfmapi_*の整理（たぶん必要なくなった)
  [
   :zabbix_user, :zabbix_password, :zabbix_server, :zabbix_url,
   :sidfmapi_user, :sidfmapi_password,
   :sidfmdlm_license_key, :sidfmdlm_license_apikey,
   :sidfmdlm_register_apikey,
  ].each do |name|
    define_singleton_method(name) { get_value(name.to_s) }
    set_name = (name.to_s + '=').to_sym
    define_singleton_method(set_name) {|value| set_value(name, value) }
  end

  def self.salt
    sidfmdlm_license_key ? "SFTK#{sidfmdlm_license_key}SFTK" : 'SFTKSFTKSFTK'
  end

  def self.sidfmdlm_register_apikey_update
    self.sidfmdlm_register_apikey = SofTekUtils.hexdigest(post: self.salt)
  end

  def self.check_sidfmdlm_register_apikey(apikey)
    self.sidfmdlm_register_apikey ? self.sidfmdlm_register_apikey == apikey : false
  end

  def self.prodver_last_update
    ret = nil
    begin
      lu = get_value(:prodver_last_update)
      ret = Time.parse(lu) if lu
    rescue Exception => e
      logger.error(e.message)
      if e.backtrace then
        e.backtrace.each do |i|
          logger.error(i)
        end
      end
    end
    ret
  end

  def self.prodver_last_update=(v)
    set_value(:prodver_last_update, v.to_s)
  end

  def self.corresp_required_sri
    ret = get_value(:corresp_required_sri)
    if ret then
      ret = ret.to_i
    else
      # High
      ret = SidfmdIssueSri::VALUE_HIGH
      set_value(:corresp_required_sri, ret.to_s)
    end
    ret
  end

  def self.corresp_required_sri=(v)
    set_value(:corresp_required_sri, v.to_s)
  end

  def self.corresp_required_cvss
    ret = get_value(:corresp_required_cvss)
    if ret then
      ret = ret.to_f
    else
      # CVSS >= 9.0
      ret = 9.0
      set_value(:corresp_required_cvss, ret.to_s)
    end
    ret
  end

  def self.corresp_required_cvss=(v)
    set_value(:corresp_required_cvss, v.to_s)
  end

  def self.corresp_required_risk
    ret = get_value(:corresp_required_risk)
    if ret then
      ret = ret.to_i
    else
      # High
      ret = 3
      set_value(:corresp_required_risk, ret.to_s)
    end
    ret
  end

  def self.corresp_required_risk=(v)
    set_value(:corresp_required_risk, v.to_s)
  end

  def self.corresp_required_area
    ret = get_value(:corresp_required_area)
    if ret then
      ret = ret.to_i
    else
      # all
      ret = 2
      set_value(:corresp_required_area, ret.to_s)
    end
    ret
  end

  def self.corresp_required_area=(v)
    set_value(:corresp_required_area, v.to_s)
  end

  def self.corresp_required_month
    ret = get_value(:corresp_required_month)
    if ret then
      ret = ret.to_i
    else
      # 1 month
      ret = 1
      set_value(:corresp_required_month, ret.to_s)
    end
    ret
  end

  def self.corresp_required_month=(v)
    set_value(:corresp_required_month, v.to_s)
  end

  def self.corresp_required
    {
      :sri => self.corresp_required_sri,
      :cvss => self.corresp_required_cvss,
      :risk => self.corresp_required_risk,
      :area => self.corresp_required_area,
      :month => self.corresp_required_month,
    }
  end

  DEFAULT_MAX_RULES = 100
  DEFAULT_MAX_RULE_LOGS = 10

  def self.max_rules
    ret = get_value(:max_rules)
    if ret then
      ret = ret.to_i
    else
      ret = DEFAULT_MAX_RULES
      set_value(:max_rules, ret.to_s)
    end
    ret
  end

  def self.max_rules=(v)
    set_value(:max_rules, v ? v.to_s : DEFAULT_MAX_RULES)
  end

  def self.max_rule_logs
    ret = get_value(:max_rule_logs)
    if ret then
      ret = ret.to_i
    else
      ret = DEFAULT_MAX_RULE_LOGS
      set_value(:max_rule_logs, ret.to_s)
    end
    ret
  end

  def self.max_rule_logs=(v)
    set_value(:max_rule_logs, v ? v.to_s : DEFAULT_MAX_RULE_LOGS)
  end

  VM_CONFIG = {}

  def self.vm_config
    if VM_CONFIG.has_key?(:instance) then
      VM_CONFIG[:instance]
    else
      VM_CONFIG[:instance] = SIDfm::VMConfig.new
      VM_CONFIG[:instance]
    end
  end

  def self.sidapi(hostname = nil, hostapikey = nil)
    api = nil
    if hostname then
      api = SIDfm::SIDAPI.new(vm_config.sidapi_url, hostname, hostapikey)
    else
      api = SIDfm::SIDAPI.new(vm_config.sidapi_url)
    end
    raise ArgumentError, 'create api failure' unless api
    if block_given? then
      begin
        yield api
      ensure
        api.close
      end
    end
    api
  end

end
