/* Copyright (C) 2018- SofTek Systems, Inc. */

// 配列同士の比較
Array.prototype.compare = function(e) {
    if (!(e instanceof Array)) {
	throw new TypeError('argument type is not array.');
    }
    var a = JSON.stringify(this.sort());
    var b = JSON.stringify(this.sort());
    return (a === b);
};

// svg のエレメントを返す。
// 必ずこれを使用すること。jqueryでエレメントを生成すると、Google Chrome以外で出力されないため。
function $s(elem) {
    return $(document.createElementNS('http://www.w3.org/2000/svg', elem));
}

// 初期化
function init_sidfm() {
    logging_error();
    enable_bootstrap_actions(null);
    html_title_update();
    force_click_ignore();
    header_title_update();
    //bootstrapのpopoverを、modalが開くときに閉じる。
    $('.modal').on('show.bs.modal', function(){
        $('[data-toggle="popover"]').popover('hide');
    });
}

// 指定したtarget配下についてtooltip, popoverを有効にする。
// targetに!targetがtrueになる値(null等)を設定すると、全体が対象となる。
function enable_bootstrap_actions(target) {
    if (!target) { target = ''; } else { target = target + ' '; }
    $(target + '[data-toggle="datepicker"]').datepicker({
	currentText: '今日',
	monthNames: [ '1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月' ],
	monthNamesShort: [ '1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月' ],
	dayNames: [ '日曜日', '月曜日', '火曜日', '水曜日', '木曜日', '金曜日', '土曜日' ],
	dayNamesShort: [ '日', '月', '火', '水', '木', '金', '土' ],
	dayNamesMin: [ '日', '月', '火', '水', '木', '金', '土' ],
	weekHeader: '週',
	dateFormat: 'yy/mm/dd',
	firstDay: 0,
	showButtonPanel: true,
	showMonthAfterYear: true,
	yearSuffix: '年',
	closeText: '閉じる'
    });
    // datepickerの「閉じる」ボタンでメニューまで閉じないようにする。
    $('#ui-datepicker-div').on('click', function(e) {
	e.stopPropagation();
	return false;
    });
    $(target + '[data-toggle="tooltip"]').tooltip();
    $(target + '[data-toggle="popover"]').popover();
}

// paginator用アクション
function init_paginator(base = 'pages') {
    $('.' + base + '_change_offset_paging').on('click', function() {
	$('#' + base + '_offset').val($(this).data('offset'));
	$('#' + base + '_form').submit();
	return false;
    });
    $('.' + base + '_change_per_page').on('change', function() {
	$('#' + base + '_per_page').val($(this).val());
	$('#' + base + '_form').submit();
	return false;
    });
    $('.' + base + '_change_offset_direct').on('keypress', function(e) {
	if (e.which == 13) {
	    var offset = parseInt($(this).val()) - 1;
	    //if (offset < 0) { offset = 1; }
	    if(offset < 0) { offset = 0; }
	    $('#' + base + '_offset').val(String(offset));
	    $('#' + base + '_form').submit();
	    return false;
	}
	return true;
  
    });
}

// sidebarについて、指定したidのアイテムをactiveにする
function activate_sidebar_item(id) {
  $('.nav-sidebar li').removeClass('active');
  $('#' + id).addClass('active');
}

// 選択された項目を別の項目に移動する。
// 2ペインのマルチセレクト入力用
//   p_selected_ids         : 選択された項目IDの一覧を保存している場所。hidden fieldのvalue等を想定。
//   p_current_selected     : 選択された項目のあるselectの場所。
//   p_opposite_selected    : 反対側の項目のあるselectの場所。
//   flg_reverse            : falseなら選択されたidを追加、trueなら選択されたidを除外。
function move_selected_items(p_selected_ids, p_current_selected, p_opposite_selected, flg_reverse) {
    var selected_ids = JSON.parse($(p_selected_ids).val());
    var options = $(p_current_selected + ' > option:selected');
    for (var i = 0; i < options.length; i++) {
	var option = $(options[i]);
	var s_id = parseInt(option.val());
	var s_text = option.text();
	if (flg_reverse) {
	    selected_ids = selected_ids.filter(function(x, i, self) {
		return x != s_id;
	    });
	    selected_ids = selected_ids.filter(function(x, i, self) {
		return self.indexOf(x) == i;
	    });
	} else {
	    selected_ids.push(s_id);
	    selected_ids = selected_ids.filter(function(x, i, self) {
		return self.indexOf(x) == i;
	    });
	}
	$(p_selected_ids).val(JSON.stringify(selected_ids));
	$(p_opposite_selected).append($('<option></option>', { value: s_id, text: s_text, class: 'text-truncate g-max-width-21em' }));
    }
    $(p_current_selected + ' > option:selected').remove();
    
    return false;
}

// heatmapの出力。
// width, height はpx単位。
// width=360, height=180で10000件程度なら問題なく表示できることを確認。
// targetはid等の文字列(内部で$()で解釈します)、dataはjqueryのエレメント
// になることに注意。
function heatmap(target, width, height, data, callback) {
    if (width <= 0 || height <= 0) {
	return;
    }
    var container = $('<div></div>');
    var svg = $s('svg').attr({
	xmlns: 'http://www.w3.org/2000/svg',
	'xmlns:xlink': 'http://www.w3.org/1999/xlink',
        preserveAspectRatio: 'xMinYMin meet',
	viewBox: '0 0 ' + width + ' ' + height,
	x: '0px',
	y: '0px',
	width: '' + width + 'px',
	height: '' + height + 'px',
	version: '1.1'
    });
    container.append(svg);
    var rect_x = Math.floor(width * (3.0/100.0));
    var rect_y = Math.floor(width * (3.0/100.0));
    var rect_width = width - (rect_x * 2);
    var rect_height = height - (rect_y);
    svg.append($s('rect').attr({
	x: '' + rect_x + 'px',
	y: '' + rect_y + 'px',
	fill: '#f8f8f8',
	width: '' + rect_width + 'px',
	height: '' + rect_height + 'px'
    }));
  /*  var x_grid_count = Math.floor(Math.sqrt(data.length * (rect_width / rect_height)));
    var y_grid_count = Math.floor(data.length / x_grid_count) + 1;
    var x_grid_size = rect_width / x_grid_count;
    if (x_grid_size > ((0.0 + rect_width) * 30.0 / 100.0)) { x_grid_size = Math.floor((0.0 + rect_width) * 30.0 / 100.0); }
    var y_grid_size = rect_height / y_grid_count;
    if (y_grid_size > ((0.0 + rect_height) * 60.0 / 100.0)) { y_grid_size = Math.fllor((0.0 + rect_height) * 60.0 / 100.0); } */
    if(rect_width >= rect_height){
        var x_grid_count = Math.ceil(Math.sqrt(data.length) * (rect_width / rect_height));
        var y_grid_count = Math.floor(data.length / x_grid_count) + 1;
        var x_grid_size = rect_width / x_grid_count;
        var y_grid_size = x_grid_size;
    }else{
        var x_grid_count = Math.ceil(Math.sqrt(data.length) * (rect_width / rect_height));
        var y_grid_count = Math.floor(data.length / x_grid_count) + 1;
        var x_grid_size = rect_height / y_grid_count;
        x_grid_count = Math.floor(rect_width / x_grid_size);
        var y_grid_size = x_grid_size;
    } 

    for (var i = 0; i < data.length; i++) {
	var x_grid_position = i % x_grid_count;
	var y_grid_position = Math.floor(i / x_grid_count);

	var x_grid_left = rect_x + x_grid_size * x_grid_position;
	var y_grid_top = rect_y + y_grid_size * y_grid_position;
        var callback_result = callback.call(this, data.get(i));
	var link = $s('a').attr({
            'xlink:href': callback_result.link,
            'data-toggle': 'tooltip',
            'data-container': 'body',
            'data-placement': 'top',
            'data-trigger': 'hover',
            'data-html': 'true',
            'title': callback_result.tooltip
	});

	var rect_opts = {
            x: '' + x_grid_left + 'px',
            y: '' + y_grid_top + 'px',
            fill: callback_result.color,
            width: '' + x_grid_size + 'px',
            height: '' + y_grid_size + 'px'
	};

	// データが少ない場合のみ枠線あり。
	if (data.length < 2000) {
	    rect_opts['stroke'] = 'black';
	    rect_opts['stroke-width'] = '0.5px';
	}

	link.append($s('rect').attr(rect_opts));
	svg.append(link);
    }

    // このようにしないとsvgが解釈されない。
    $(target).html(container.html());
    enable_bootstrap_actions(target);
}

// グラディエーションの計算
// s, eはそれぞれ変換の始端・終端の色をHashで指定する。
//   { r: 1, g: 2, b: 3 }
// pは変化の割合(%)
function calc_gradient_color(s, e, p) {
    var ret = { r:0, g:0, b:0 };
    var keys = [ 'r', 'g', 'b' ];
    for (var i = 0; i < keys.length; i++) {
      ret[keys[i]] = Math.floor((e[keys[i]] - s[keys[i]]) * p / 100.0 + s[keys[i]]);
    }

    return ret;
}

// hex形式の色をcalc_gradient_colorでで使用する形式に変換
function from_hex_to_rgb(hex) {
    var res = ('' + hex).match(/^#([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/);
    var r = parseInt(res[1], 16);
    var g = parseInt(res[2], 16);
    var b = parseInt(res[3], 16);
    return { r:r, g:g, b:b };
}

// calc_gradient_colorで返すrgbのHashを色指定用の文字列(rgb)に変換
function to_rgb_str(s) {
    return 'rgb(' + s.r + ',' + s.g + ',' + s.b + ')';
}

// calc_gradient_colorで返すrgbのHashを色指定用の文字列(hex)に変換
function to_rgb_hex(s) {
    return '#' +
	[ 'r', 'g', 'b' ].map(function(i) {
	    return ('0' + s[i].toString(16)).slice(-2);
	}).join('');
}

// color_引数のidが存在する場合に、その値を取得して返す。
function symbol_color(color) {
    var c = $('#color_' + color);
    return c.length ? c.val() : c;
}

// SidfmHelper#status_barとほぼ同じ動作。
// modal のように動的処理で生成する必要のある場所で採用。   
function status_bar(width, height, data, unit, opts) {
    opts = (typeof opts === 'undefined') ? {} : opts;
    opts['width'] = '' + width + 'px';
    opts['height'] = '' + height + 'px';
    opts['xmlns'] = 'http://www.w3.org/2000/svg';
    opts['version'] = '1.1';
    var svg = $s('svg').attr(opts);
    var data_size = data.length;
    var value_sum = 0;
    for (var i = 0; i < data_size; i++) {
	value_sum += data[i]['value'];
    }
    var unit_width = parseFloat(width) / parseFloat(value_sum);
    var current_x = 0;
    for (var i = 0; i < data_size; i++) {
	var w = Math.floor(unit_width * parseFloat(data[i]['value']));
	var html = "<svg xmlns='http://www.w3.org/2000/svg' version='1.1' width='12px' height='12px'><rect x='0' y='0' width='10' height='12' fill='" + symbol_color(data[i]['color']) + "' /></svg>" + data[i]['name'] + ":" + data[i]['value'] + unit;
	var a_opt = {
	    'data-toggle': 'tooltip',
	    'data-container': 'body',
	    'data-placement': data[i]['placement']  || 'bottom',
	    'data-html': 'true',
	    title: html
	};
	var a = $s('a').attr(a_opt);
	svg.append(a);
	var rect_opt = {
	    x: '' + current_x,
	    y: '0',
	    width: '' + w,
	    height: '' + height,
	    fill: symbol_color(data[i]['color'])
	};
	a.append($s('rect').attr(rect_opt));
	current_x += w;
    }
    return svg;
}

// QUERY_STRINGを考慮して現在のURLにパラメータを付与する
function append_link(append_opt) {
    var ret = location.href;
    if (ret.indexOf('?') >= 0) {
	ret += '&' + append_opt;
    } else {
	ret += '?' + append_opt;
    }
    return ret;
}

// Stacked Fill Graphで使用。yの値にbaseのyの値を加算して返す。
function calc_stacked(base, stack) {
    for (var i = 0; i < base['y'].length; i++) {
	stack['y'][i] += base['y'][i];
    }
    return stack;
}

// 指定した箇所の幅を取得し、グラフの幅を計算して返す。
function graph_width(target) {
    return $(target).innerWidth() - 10;
}

//
function url_replace_id(target, id) {
    return $(target).val().replace('___id___', '' + id);
}

// 指定されたidのホスト詳細画面のURLを返す。
function url_host_detail(id) {
    return url_replace_id('#sidfm_host_path', id);
}

// 指定されたidの脆弱性詳細画面のURLを返す。
function url_vulnerabilities_detail(id) {
    return url_replace_id('#sidfm_vulnerability_path', id);
}
function url_vulnerability_detail(id) {
    return url_replace_id('#sidfm_vulnerability_path', id);
}

//
function url_info_product_detail(id) {
    return url_replace_id('#sidfm_informations_product_path', id);
}

//
function url_info_vuln_detail(id) {
    return url_replace_id('#sidfm_informations_vuln_path', id);
}

// <title></title>の文字列にSIDfm VMが含まれていたら、SIDfm&trade; VM
// に変更する
function html_title_update() {
    $('title').html($('title').html().replace('SIDfm', 'SIDfm&trade;'));
}

// 指定した要素にclickに反応しないようにする。
// FIXME: 既にclickイベントが登録されている場合への対応
function force_click_ignore() {
    $('.ignore-click').on('click', function() {
	return false;
    });
}

// ソート処理
function sort_toggle(name, base = 'pages') {
    return (function() {
	var order = $('#' + base + '_order').val();
	if (order == name + '_asc') {
	    $('#' + base + '_order').val(name + '_desc');
	} else {
	    $('#' + base + '_order').val(name + '_asc');
	}
	$('#' + base + '_form').submit();
	return false;
    });
}

// URLやパスから、anchor (fragment) を削除する
function remove_anchor(str) {
    return (str ? str.replace(/#[0-9a-zA-z_-]+$/, '') : '');
}

// ヘッダのタイトルを更新する。
function header_title_update(title) {
    var header_title = $('#sidfm_header_title');
    var vm_page_title = $('#vm_page_title').text();
    header_title.text(vm_page_title);
    header_title.removeClass('d-none');
}

// windows.onerrorでアプリケーションにロギングする
// なお、ログイン状態でないとロギングに失敗する（仕様）
function logging_error() {
    window.onerror = function(msg, file, line, column, err) {
	if (!msg) { msg = '(no msg)'; }
	if (!file) { file = '(no file)'; }
	if (!line) { line = '(unknown)'; }
	if (!column) { column = 0; }
	if (!err) { err = null; }
	
	var logger_path = $('#sidfm_internal_api_logger_path').val();
	if (logger_path) {
	    $.post(logger_path,
		   { 'params[msg]': msg,
		     'params[file]': file,
		     'params[column]': column,
		     'params[line]': line,
		     'params[err]': err },
		   function(data) {
		       console.log('msg: ' + msg);
		       console.log('file: ' + file);
		       console.log('column: ' + column);
		       console.log('line: ' + line);
		       console.log('err: ' + err);
		       console.log('data:' + data);
		   });
	}
	return false;
    };
}

function sticky_control(table_name, page_name) {
  window.onscroll = function() { stickyAdd(); };
  var wrap_ActionArea = document.getElementById("wrap-ActionArea");
  var target = document.getElementById(table_name);
  const footer = $('footer');
  $('#ActionArea').css('margin-left','0');
  function stickyAdd() {
      var calc_offset = function(){
                            var offset = 0;
                            if(page_name === 'hosts' || page_name === 'host' || page_name === 'vulnerability' || page_name === 'rule'){
                                var SummaryT  = document.getElementById('SummaryTable');
                                var SummaryTdynamic = SummaryT.offsetTop;
                                var SummaryTHeight= document.getElementById('SummaryTable').clientHeight;
                                offset = SummaryTdynamic + SummaryTHeight;
                             }else if(page_name === 'vulnerabilities'){
                                 var SummaryT  = document.getElementById('SummaryTable');
                                 var SummaryStaticOffset = SummaryT.offsetTop;
                                 var SummaryTHeight= document.getElementById('SummaryTable').clientHeight;
                                 var accordion1_h  = document.getElementById('system_category_table').clientHeight;
                                 var accordion2_h  = document.getElementById('OS_distributions').clientHeight;
                                 var accord_h = accordion1_h + accordion2_h;
                                 offset = SummaryStaticOffset + accord_h + SummaryTHeight;
                             }else if(page_name ==='users' || page_name === 'licence'){
                                 var clientRect = target.getBoundingClientRect();
                                 offset = clientRect.top; 
                             }
                             return offset;
                         };
      if (window.pageYOffset >= calc_offset()) {
           if( $(wrap_ActionArea).find('footer').length === 0){
               $(wrap_ActionArea).append(footer);
           }
           $('#hidden-space').height($('#wrap-ActionArea').height());
           wrap_ActionArea.classList.add('sticky');
           //$('[data-toggle=modal]').on('click', function(e){
           $('.modal').on('show.bs.modal', function(){
               $(wrap_ActionArea).css('z-index', '10');
           });
           $('.modal').on('hidden.bs.modal', function(){
               if($('[data-slidemodal]:visible').length === 0){
                   $(wrap_ActionArea).css('z-index', '1045');
               }else{
                   $(wrap_ActionArea).css('z-index', '500');
               }
           }); 
      } else {
          $('#hidden-space').height('0');
          wrap_ActionArea.classList.remove('sticky');
      }
  }
}

function set_scroll(){
    var ScrollTable = $('[data-outertable="scroll"]');
    var inner_table = $('[data-innertable="scroll"]');
    var ScrollTable_offset = ScrollTable.offset().top;
    var MainContainer_bottom = parseInt($('#MainContainer').css('padding-bottom').replace('px', ''));
    var space_for_scroll =  (MainContainer_bottom > 0 ? MainContainer_bottom : 35)
                            + parseInt($('#ActionArea').css('padding').replace('px', ''))
                            ;
    var doc_height = $(document).innerHeight();
    var window_height = $(window).innerHeight();
    var scrolled_quantity = $(window).scrollTop();
    var bottom_height = $('#wrap-ActionArea').hasClass('sticky') ?
                        ($('#wrap-ActionArea').outerHeight(true)
                         - ($('#pagenationArea').outerHeight(true) + space_for_scroll))
                         :
                         ($('#ActionArea').outerHeight(true) + $('footer').outerHeight(true)
                         + $('#pagenationArea').outerHeight(true))
                         ;
    if((doc_height - window_height) > (scrolled_quantity + bottom_height) 
         && ((ScrollTable_offset - window_height + 15) < scrolled_quantity)
         && (ScrollTable.width() < ScrollTable.get(0).scrollWidth)
         && ($('#floating-scroll').length === 0)){
         var dummy_scroll = $('<div>')
                            .attr('id', 'floating-scroll')
                            .css({
                                 'left': ScrollTable.offset().left,
                                 'bottom': $('#wrap-ActionArea').hasClass('sticky') ? $('#wrap-ActionArea').height() : 0
                                 });
         var inner_dummy_scroll = $('<div>')
                                   .attr('id', 'inner-floating-scroll')
                                   .css({
                                       'height':'20px',
                                       'z-index':'1060'
                                       });
         dummy_scroll.append(inner_dummy_scroll);
         $('#content-block').append(dummy_scroll);
         dummy_scroll.on('scroll', function(){
             ScrollTable.get(0).scrollLeft = dummy_scroll.get(0).scrollLeft;
         });
    }
    if(!$('#wrap-ActionArea').hasClass('sticky')){
        $('#floating-scroll').css('bottom', '0');
    }else{
        $('#floating-scroll').css('bottom',  $('#wrap-ActionArea').height());
    }
    if((doc_height - window_height) > (scrolled_quantity + bottom_height) 
        && ((ScrollTable_offset - window_height + 15) < scrolled_quantity)){
        $('#floating-scroll').show();
    }else{
        $('#floating-scroll').hide();
    }
    $('#floating-scroll').offset({'left':ScrollTable.offset().left});
    $('#floating-scroll').width(ScrollTable.width());
    $('#inner-floating-scroll').width(inner_table.width());
}

//ホスト名；カテゴリ名を表示するバルーンの表示
function enable_dropdown_balloon(){
    $('.create-balloon').on('click', function(){
        var trigger_btn = $(this);
        var trigger_parent = trigger_btn.closest('.balloon-dropdown');
        var max_width = trigger_parent.find('.balloon-menu').css('max-width').replace('px', '');
        var is_space_of_right = $(window).width() - (trigger_btn.offset().left + trigger_btn.outerWidth()) > max_width;
        trigger_parent.removeClass('dropright dropleft').addClass(is_space_of_right ? 'dropright' : 'dropleft');
        trigger_parent.removeClass('balloon-right balloon-left')
            .addClass(is_space_of_right ? 'balloon-right' : 'balloon-left');
        trigger_btn.attr({
            'data-toggle': 'dropdown',
            'data-boundary': 'window',
            'aria-haspopup': 'true',
            'aria-expanded': 'false',
            'data-flip': 'false',
        }).dropdown();
    });
}
        

