#!/usr/bin/ruby
## vim: set ts=2 sw=2 expandtab fileencoding=utf-8 ai :

Encoding.default_external="UTF-8"

require 'erb'
require 'json'

class GenerateAnsiblePlaybook

  RE_RPM_OS = /Red Hat|CentOS|Amazon/i
  RE_DEB_OS = /Ubuntu|Debian/i
  RE_WIN_OS = /Windows/i

  TEMPLATE_APT_PLAYBOOK=<<-'EOF'
- name: update deb packages
  hosts: <%= @hostname %>

  tasks:
    - name: apt upgrade
      apt:
        name:
<%      @affect_package_names.each {|pkg_name|  -%>
          - <%= pkg_name %>
<%      }  -%>
        state: latest
        update_cache: yes
        dpkg_options: 'force-confold'
      register: r_apt_upgrade
      become: true

    - name: output apt upgrade
      debug:
        var: r_apt_upgrade

EOF
  
  TEMPLATE_YUM_PLAYBOOK=<<-'EOF'
- name: update rpm packages
  hosts: <%= @hostname %>

  tasks:
    - name: yum update
      yum:
        name:
<%      @affect_package_names.each {|pkg_name|  -%>
          - <%= pkg_name %>
<%      }  -%>
        state: latest
      register: r_yum_update
      become: true

    - name: output yum update
      debug:
        var: r_yum_update

EOF

  TEMPLATE_WIN_UPDATES_PLAYBOOK=<<-'EOF'
- name: Windows Updates
  hosts: <%= @hostname %>

  tasks:
    - name: windows updates
      win_updates:
        category_names:
          - SecurityUpdates
          - CriticalUpdates
          - UpdateRollups
          # - DefinitionUpdates
          # - DeveloperKits
          # - FeaturePacks
          # - ServicePacks
      register: r_win_update

    - name: output win update
      debug:
        var: r_win_update

EOF

  def initialize(data)
    @data = data
    @hostname = ''
    @os = ''
    @affect_package_names = []
  end

  def parse_json(res)
    res_json = JSON.parse(res)
    @hostname = res_json['host']['hostname']
    @os = res_json['host']['os']
    res_json['content'].each {|content|
      content['affect_packages'].each {|aff_pkg|
        @affect_package_names.push(aff_pkg['name'])
      }
    }
    @affect_package_names.sort!
    @affect_package_names.uniq!
  end


  def gen_win_playbook
    erb = ERB.new(TEMPLATE_WIN_UPDATES_PLAYBOOK, nil, '-')
    playbook = erb.result(binding)
    return playbook
  end


  def gen_apt_playbook
    erb = ERB.new(TEMPLATE_APT_PLAYBOOK, nil, '-')
    playbook = erb.result(binding)
    return playbook
  end


  def gen_yum_playbook
    erb = ERB.new(TEMPLATE_YUM_PLAYBOOK, nil, '-')
    playbook = erb.result(binding)
    return playbook
  end


  def generate
    json_data = @data
    if !json_data then
      p = 'error'
    else
      parse_json(json_data)
      if RE_WIN_OS =~ @os then
        p = gen_win_playbook
      else
        unless @affect_package_names.size > 0 then
          return 'no_packages'
        end
        if RE_RPM_OS =~ @os
          p = gen_yum_playbook
        elsif RE_DEB_OS =~ @os
          p = gen_apt_playbook
        end
      end
    end
    return p
  end

end
