#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require File.expand_path('./api_base.rb', File.dirname(__FILE__))

require 'time'

module SIDfm

  class SIDAPI<APIBase

    # デバッグモード有効
    def self.is_debug?
      FileTest.exist?('/flag.debug')
    end

    private

    def initialize(base_url = nil, hostname = nil, hostapikey = nil)
      super(base_url)
      @hostname = hostname
      @hostapikey = hostapikey
      @licenseid = Sidfm.sidfmdlm_license_key
      @licenseapikey = Sidfm.sidfmdlm_license_apikey
    end

    def create_request_headers
      header = super
      header['X-Hostname'] = @hostname if @hostname
      header['X-Host-Api-Key'] = @hostapikey if @hostapikey
      if @hostname.nil? and @hostapikey.nil? then
        header['X-License-Id'] = @licenseid if @licenseid
        header['X-Api-Key'] = @licenseapikey if @licenseapikey
      end
      header
    end

    public

    def test
      api_request('test', method: :get)
    end

    def test_with_auth
      api_request('test_with_auth', method: :get)
    end

    def products_vulns(product_ids: [], product_version_ids: [], packages: '', begin_datetime: '2017/06/26T00:00:00', end_datetime: '2017/07/02T00:00:00', time_field: '', package_scan_period: 'all')
      d = { :product_ids => product_ids, :product_version_ids => product_version_ids, :packages => packages, :begin_datetime => begin_datetime, :end_datetime => end_datetime, :time_field => time_field, :package_scan_period => package_scan_period, }
      api_request('products_vulns', method: :post, body: JSON.generate(d))
    end

    def package_support_products
      api_request('package_support_products', method: :get)
    end

    def areas
      api_request('areas', method: :get)
    end

    def risks
      api_request('risks', method: :get)
    end

    # deprecated
    def products(begin_datetime = nil, end_datetime = nil)
      d = if begin_datetime.nil? or end_datetime.nil? then
            {}
          else
            if begin_datetime.is_a?(Time) then
              begin_datetime = begin_datetime.to_datetime
            end
            if begin_datetime.is_a?(DateTime) then
              begin_datetime = begin_datetime.to_s
            end
            if end_datetime.is_a?(Time) then
              end_datetime = end_datetime.to_datetime
            end
            if end_datetime.is_a?(DateTime) then
              end_datetime = end_datetime.to_s
            end
            { 'begin_datetime' => begin_datetime, 'end_datetime' => end_datetime }
          end
      api_request('products.json', method: :get, begin_datetime: d['begin_datetime'], end_datetime: d['end_datetime'])
    end

    def vuln(sidfmid)
      d = { :id => sidfmid }
      api_request('vuln.json', method: :post, body: JSON.generate(d))
    end

    def find_news_topics(params = nil)
      api_request('find_news_topics', method: :post, body: params ? JSON.generate(params) : '')
    end

    def find_products(params = nil)
      api_request('find_products', method: :post, body: params ? JSON.generate(params) : '')
    end

    def recent_product_eols(params = nil)
      api_request('recent_product_eols', method: :post, body: params ? JSON.generate(params) : '')
    end

    # productsと同じAPIにアクセスするが、引数の与え方が異なる。
    def products2(params = nil)
      api_request('products', method: :post, body: params ? JSON.generate(params) : '')
    end

    def product(params = nil)
      api_request('product', method: :post, body: params ? JSON.generate(params) : '')
    end

    def product_eols(params = nil)
      api_request('product_eols', method: :post, body: params ? JSON.generate(params) : '')
    end

    def product_vulns(params = nil)
      api_request('product_vulns', method: :post, body: params ? JSON.generate(params) : '')
    end

    def find_vulns(params = nil)
      api_request('find_vulns', method: :post, body: params ? JSON.generate(params) : '')
    end

    def sri_categories(params = nil)
      api_request('sri_categories', method: :post, body: params ? JSON.generate(params) : '')
    end

    # vulnと同じAPIにアクセスするが、引数の与え方が異なる。
    def vuln2(params = nil)
      api_request('vuln', method: :post, body: params ? JSON.generate(params) : '')
    end

    # MSKBから脆弱性を検索
    def find_vulns_by_mskb(kb)
      api_request('find_vulns_by_mskb', method: :get, body: nil, mskb: kb)
    end

    # htmlidの配列からshort_descriptionを返す。
    def vuln_short_descriptions(htmlids)
      dat = {
        'vuln_ids' => htmlids || [],
      }
      api_request('vuln_short_descriptions', method: :post, body: JSON.generate(dat))
    end

  end

end


if $0 == __FILE__ then

  require 'pp'
  require 'open3'

  CMD_RPM = '/usr/bin/rpm'
  CMD_SORT = '/usr/bin/sort'

  def exec_ret_array_pipe(*cmd)
    ret = []
    Open3.pipeline_r(*cmd) do |f_stdout, f_status|
      while v = f_stdout.gets do
        ret << v.chomp
      end
    end
    ret
  end

  SIDfm::SIDAPI.new('http://dev-sidapi11.softek.co.jp/v1/') do |o|

    pp o.products_vulns(product_ids: [ 'vertag_id_675', 'aplvertag_id_8538' ],
                        dt1: '2017/06/26T00:00:00', dt2: '2017/07/02T00:00:00')

    rpmlist = exec_ret_array_pipe([ CMD_RPM, '-qa' ], [ CMD_SORT ])

    # vertag_id_646 = CentOS 7
    pp o.products_vulns(product_ids: [ 'vertag_id_646' ],
                        packages: rpmlist.join(".rpm\n"),
                        dt1: '2017/07/19T00:00:00', dt2: '2017/07/27T00:00:00')

    pp o.package_support_products

    pp o.products([ 'aplication_id_1' ])

    pp o.vuln(24282)

  end

end
