#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssueCve < ActiveRecord::Base
  belongs_to :sidfmd_issue

  scope :enable_hosts, -> { joins(:sidfmd_issue).joins(:sidfmd_issue => [ :sidfmd_host ]).where(:sidfmd_hosts => { :flg_delete => false }) }

  scope :enable_categories, -> { joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => SidfmdCategoriesHost.select(:sidfmd_host_id).distinct(:sidfmd_host_id)}) }

  scope :exploitable, -> { joins(:sidfmd_issue).where('c2_exploitable IS TRUE OR c3_exploitable IS TRUE').where(:sidfmd_issues => { :sidfmd_host_id => SidfmdHost.enable_hosts.select(:id) }).group(:sidfmd_host_id) }

  NOT_REGISTERED = '未登録'.freeze
  RISK_FIXED = '済'.freeze


  # cvss{:label, :score}から出力用の文字列を返す。
  def self.display_cvss(i)
    case i[:label]
    when :notreg then NOT_REGISTERED
    else sprintf('%2.1f', i[:score])
    end
  end

  LABEL = {
    :notreg => nil,
    :d4 => { :top => 10.0 , :bottom => 9.0 }.freeze,
    :d3 => { :top => 9.0, :bottom => 7.0 }.freeze,
    :d2 => { :top => 7.0, :bottom => 4.0 }.freeze,
    :d1 => { :top => 4.0, :bottom => 0.0 }.freeze,
  }.freeze

  LABELS = [ :notreg, :d1, :d2, :d3, :d4 ].map(&:freeze).freeze
  DISPLAY_SORTED_LABELS = [ :notreg, :d4, :d3, :d2, :d1 ].map(&:freeze).freeze

  def self.label2value(label)
    idx = LABELS.index(label)
    idx ? idx : 0
  end

  # cvssスコアからラベルを取得する。
  def self.labelling(s)
    if s.nil?                                               then :notreg
    elsif                          s >= LABEL[:d4][:bottom] then :d4
    elsif s < LABEL[:d3][:top] and s >= LABEL[:d3][:bottom] then :d3
    elsif s < LABEL[:d2][:top] and s >= LABEL[:d2][:bottom] then :d2
    else                                                         :d1
    end
  end

end
